<?php
/*                                                                                                                 
Plugin Name: 410 for Wordpress
Plugin URI: http://rayofsolaris.net/code/410-for-wordpress
Description: Sends HTTP 410 (Gone) responses to requests for pages that no longer exist on your blog.
Version: 0.2
Author: Samir Shah
Author URI: http://rayofsolaris.net/
License: GPL2
Text Domain: wp-410
*/

if( !defined('ABSPATH') ) exit;

class WP_410 {
	const dom = 'wp-410';
	const link_opt = 'wp_410_links_list';
	const version_opt = 'wp_410_options_version';
	const db_version = 1; 	// DB version, since 0.2
	private $permalinks;
	
	function __construct() {
		$struct = get_option('permalink_structure');
		$this->permalinks = !empty( $struct );
		load_plugin_textdomain( self::dom, false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
		
		if( get_option( self::version_opt, 0 ) < self::db_version )
			add_action( 'init',			array( &$this, 'upgrade' )				);
		
		if( is_admin() )
			add_action( 'admin_menu',	array( &$this, 'settings_menu' ) 		);
		
		add_action('delete_post',		array( &$this, 'note_deleted_post' )	);
		add_action('wp_insert_post', 	array( &$this, 'note_inserted_post' )	);
		add_action('template_redirect', array( &$this, 'check_for_410' ) 		);
	}
	
	function upgrade() {
		// upgrade from 0.1 to 0.2
		$old_links = get_option( self::link_opt, array() );
		$new_links = array();
		foreach( $old_links as $link )
			$new_links[$link] = $this->build_regex( $link );	// new links have key = display link and value = regex

		update_option( self::link_opt, $new_links );
		update_option( self::version_opt, self::db_version );
	}
	
	function settings_menu() {
		add_submenu_page('plugins.php', __('410 for Wordpress', self::dom), __('410 for Wordpress', self::dom), 'manage_options', 'wp_410_settings', array(&$this, 'settings_page') );
	}
	
	function settings_page() {
		$old_links = get_option( self::link_opt, array() );
		
		if ( isset( $_POST['submit'] ) ) {
			$failed_to_add = array();
			if( !empty( $_POST['old_links_to_add'] ) ) {
				$added_links = preg_split( '/(\r?\n)+/', $_POST['old_links_to_add'], -1, PREG_SPLIT_NO_EMPTY );
				
				foreach( $added_links as $k => $link ) {
					$parts = parse_url( $link );
					if( !$parts || empty( $parts['scheme'] ) || empty( $parts['host'] ) ) // invalid URL
						$failed_to_add[] = '<code>' . htmlspecialchars( $link ) . '</code>';
					else
						$old_links[$link] = $this->build_regex( $link );	// key is display link, value is regex
				}
			}
			
			if( !empty( $_POST['old_links_to_remove'] ) )
				foreach( $_POST['old_links_to_remove'] as $key )
					unset( $old_links[$key] );
			
			ksort( $old_links );
	
			update_option( self::link_opt, $old_links );
			echo '<div class="updated fade"><p>'.__('Options updated.', self::dom).'</p></div>';
			
			if( $failed_to_add )
				echo '<div class="error"><p>'. __('The following entries could not be recognised as valid URLs, and were not added to the list.', self::dom). '<br />- ' . implode( '<br />- ', $failed_to_add ) .'</p></div>';
		}
	?>
	<style>
	.indent {padding-left: 2em}
	.wp_gone_deleted label {text-decoration: line-through}
	</style>
	<div class="wrap">
	<h2><?php _e('410 for Wordpress', self::dom);?></h2>
	<p><?php _e('This plugin will issue a HTTP 410 response to articles that no longer exist on your blog. When an article is deleted, it records this and issues a 410 response when pages unique to that article are requested. This HTTP code informs robots visiting your site that the requested page has been permanently removed, and that they should stop trying to access it.', self::dom);?></p>
	<p><?php _e('Note: this plugin will only return a 410 response if Wordpress has not found something valid to display for the requested URL.', self::dom);?></p>
	<form action="" method="post" id="wp-gone-captcha-settings">
	<h3><?php _e('Obsolete URLs', self::dom);?></h3>
	<?php
	if( empty( $old_links ) ) {
		echo '<p>' . __( 'There are currently no obsolete URLs in this list. You can add some manually below.', self::dom ) . '</p>';
	}
	else {
		echo '<p>' . __( 'The following URLs are currently considered obsolete. To remove items from the list, check the corresponding box.', self::dom ) .'</p>';
		echo '<ul id="wp_gone_old_links" class="indent">';
		foreach( array_keys( $old_links ) as $k )
			echo "<li><input type='checkbox' name='old_links_to_remove[]' value='" . htmlspecialchars( $k ) . "' id='old_link_$k' /> <label for='old_link_$k'><code>" . htmlspecialchars( $k ). "</code></label></li>";
		echo '</ul>';
	}
	?>
	<p><small><?php _e('If you create or update an article whose URL matches one of those above, that URL will automatically be removed from this list.', self::dom);?></small></p>
	<h3><?php _e('Manually add URLs', self::dom);?></h3>
	<p><?php _e('You can manually add items to the list by entering them below. Please enter one <strong>fully qualified</strong> URL per line.', self::dom);?></p>
	<p><?php printf( __( 'Use %s as a wildcard character. So %s will match all URLs ending in %s.'), '<code>*</code>', '<code>http://www.example.com/*/music/</code>', '<code>/music/</code>' );?></p>
	<textarea name="old_links_to_add" rows="8" cols="80"></textarea>
	<p class="submit"><input class="button-primary" type="submit" name="submit" value="<?php _e('Update settings', self::dom);?>" /></p>
	</form>
	</div>
	<script>
	jQuery(document).ready(function(){
		jQuery('#wp_gone_old_links input').change( function(){
			this.parentNode.className = jQuery(this).is(":checked") ? 'wp_gone_deleted' : '';
		});
	});
	</script>
<?php
	}
	
	function note_deleted_post( $id ) {
		// NOTE: function wp_delete_post calls the delete_post action twice
		static $delete_noted = false;
		
		if( $delete_noted )
			return;
		$delete_noted = true;
		
		$post = get_post( $id );
		
		if( 'revision' == $post->post_type )
			return;
			
		$link = get_permalink( $id );
		if( $this->permalinks )
			$link .= '*';	// catch everything like /page/ and /feed/

		$list = get_option( self::link_opt, array() );
		
		$list[$link] = $this->build_regex( $link );
		
		update_option( self::link_opt, $list );
	}
	
	function note_inserted_post( $id ) {
		$post = get_post( $id );
		
		if( 'revision' == $post->post_type || 'draft' == $post->post_status )
			return;
		
		// Check our list of URLs against the new/updated post's permalink, and if they match, scratch it from our list
		$links = array();
		
		$links[] = get_permalink( $id );
		$links[] = get_post_comments_feed_link( $id );	// back compat
				
		if( $this->permalinks )
			$links[0] .= '*';
			
		$list = get_option( self::link_opt, array() );
		
		foreach( $links as $link )	
			unset( $list[$link] );
	
		update_option( self::link_opt, $list );
	}
	
	function check_for_410() {
		// Don't mess if Wordpress has found something to display
		if( !is_404() )
			return;
			
		$links = get_option( self::link_opt, array() );
		$req  = ( is_ssl() ? 'https://' : 'http://' ) . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
		
		foreach( $links as $regex ) {
			if( @preg_match( $regex, $req ) ) {
				status_header( 410 );
				_e( 'Sorry, the page you requested has been permanently removed.', self::dom );
				exit;
			}
		}
	}
	
	private function build_regex( $link ) {
		$parts = preg_split( '/(\*)/', $link, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY );
		foreach( $parts as &$part ) if( '*' != $part )
			$part = preg_quote( $part, '|' );
		$parts = str_replace( '*', '.*', $parts );
		return '|^' . implode( '', $parts ) . '$|';
	}
}

new WP_410();
?>